// The MIT License (MIT)
// Copyright © 2022 Ivan Vorobei (hello@ivanvorobei.io)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import FirebaseFirestore
import FirebaseStorage

extension SPProfiling.Profile {
    
    enum Firestore {
        
        public static func profiles_collection() -> CollectionReference {
            let firestore = FirebaseFirestore.Firestore.firestore()
            return firestore.collection(Constants.Firebase.Firestore.Paths.profiles)
        }
        
        public static func profile_document(userID: String) -> DocumentReference {
            profiles_collection().document(userID)
        }
    }
    
    enum Storage {
        
        public static func profiles_folder() -> StorageReference {
            let storage = FirebaseStorage.Storage.storage()
            return storage.reference().child(Constants.Firebase.Storage.Paths.profiles)
        }
        
        public static func profile_avatar(userID: String) -> StorageReference {
            profiles_folder().child(userID).child(Constants.Firebase.Storage.Paths.avatar_filename)
        }
    }
}
